@@ -43,6 +43,15 @@ def get_pefastqs(fastqs: List[Path]) -> Dict[str, Dict[str, Path]]:
             )
             sys.exit(1)
 
+    for sample, reads in pefastqs.items():
+        if len(reads) != 2:
+            term.print(
+                "[MergeError]: please ensure there is and R1 and R2 for all samples"
+            )
+            term.print("[MergeError]: detected the following samples")
+            term.print(pefastqs)
+            sys.exit(1)
+
     return pefastqs
 
 
@@ -87,6 +96,7 @@ def merge_single(
         )
 
         run_cmd(command, sample, merged_barcode_fastq, verbose, status)
+        term.process("fastqs sucessfully merged")
 
 
 def merge_all(
@@ -132,7 +142,6 @@ def merge_all(
                 status,
             )
 
-        term.process("fastqs sucessfully merged")
         term.process(status="end")
 
     term.print("\n[green]FINISHED!")
